#!/usr/bin/env python

import argparse
import csv
import datetime
import json
import subprocess
import tempfile

RPPR = 'rppr'
QUIET = False

def control(path='control.json'):
    with open(path) as fp:
        return json.load(fp)

def extract_adcl(adcl_fp, leaves):
    r = csv.reader(adcl_fp)
    for l, adcl in r:
        if int(l) == leaves:
            return float(adcl)
    raise ValueError("ADCL not found for {0}".format(leaves))

def min_adcl_tree(tree_file, leaves, algorithm='pam'):
    cmd = [RPPR, 'min_adcl_tree', '--algorithm', algorithm, '--leaves', str(leaves)]
    with tempfile.NamedTemporaryFile(prefix='adcl-') as adcl_fp:
        cmd.extend(('--all-adcls-file', adcl_fp.name))
        cmd.append(tree_file)
        start = datetime.datetime.now()
        if not QUIET:
            print ' '.join(cmd)
        out = subprocess.check_output(cmd)
        end = datetime.datetime.now()
        return extract_adcl(adcl_fp, leaves), frozenset(out.splitlines()), (end - start).total_seconds()

def main(ctrl=None):
    p = argparse.ArgumentParser()
    p.add_argument('tree')
    p.add_argument('bootstrap_trees', type=argparse.FileType('r'))
    p.add_argument('keep_leaves', type=int)
    p.add_argument('outfile', type=argparse.FileType('w'))
    p.add_argument('--algorithm', choices=('pam', 'full'), default='full')
    p.add_argument('--quiet', default=False, action='store_true')

    a = p.parse_args()
    global QUIET
    QUIET = a.quiet

    actual_adcl = min_adcl_tree(a.tree, a.keep_leaves, a.algorithm)[0]

    with a.bootstrap_trees as tree_fp, a.outfile as fp:
        w = csv.writer(fp, lineterminator='\n')
        w.writerow(('i', 'k', 'adcl', 'adcl_actual', 'time'))

        for i, t in enumerate(tree_fp):
            with tempfile.NamedTemporaryFile(prefix='tree-', suffix='.nwk') as tf:
                tf.write(t)
                tf.flush()

                adcl, _, time = min_adcl_tree(tf.name, a.keep_leaves, a.algorithm)
                w.writerow((i, a.keep_leaves, adcl, actual_adcl, time))

if __name__ == '__main__':
    main()
